/********************************************************************/
/*                                                                  */
/*      Polled I/O  Program for SNDS100                             */
/*                                                                  */
/*------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                     */
/*------------------------------------------------------------------*/
/*                                                                  */
/*      Modified, programmed by qorrha (Jong H. BAIK)               */
/*                                                                  */
/********************************************************************/

/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/*      hwinit.c                   KS32C5000, KS32C50100   : version 1.0 */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/*      InitInterrupt(); //Interrupt mode & vector table initialized     */
/*      Rtc_init() ;     //Timer0 used for Rtc clock tick                */
/*                                                                       */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*      Young Sun KIM, Samsung Electronics, Inc.                         */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*      SNDS board initializing routine                                  */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*                      1997-05-12          first edited                 */
/*                      1997-06-19          modified for KS32C5000       */
/*                      1999-01-04          modified for KS32C50100      */
/*                                                                       */
/*************************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002			*/
/*			*/

#include <stdarg.h>

#define TEST_POLLIO

#include "evm50100.h"

/*---------------------------------------------------------------------*/
/* General Definitions                                                 */
/*---------------------------------------------------------------------*/
#define UCHAR   unsigned char
#define USHORT  unsigned short
#define UINT    unsigned int
#define ULONG   unsigned long

/*---------------------------------------------------------------------*/
/* Symbol Definitions                                                  */
/*---------------------------------------------------------------------*/
#define KEY_NL          0x0A
#define KEY_CR          0x0D
#define KEY_BSP         0x08
#define KEY_ESC         0x1B
#define KEY_CTRLZ       0x1A
#define KEY_RUBOUT      0x7F
#define LTRUE           0xFF
#define FALSE              0
#define NULL               0
#define DEFAULT           -1
#define DEFAULT_P   	   6      /* default precision */

/*---------------------------------------------------------------------*/
/* This structure is used to store information obtained from parsing a */
/* format specifier.                                                   */
/*---------------------------------------------------------------------*/
typedef struct
{
 long width;            /* field width */
 long precision;        /* precision specifier */
 ULONG count;           /* character count */
 /* BOOLEAN flags with default values */
 UCHAR space;           /* FALSE */
 UCHAR right;           /* TRUE */
 UCHAR alter;           /* FALSE */
 UCHAR plus;            /* FALSE */
 UCHAR zero;            /* FALSE */
 UCHAR mod;             /* process modifier */
 UCHAR type;            /* conversion type */
} FORMAT;


static void  strout(FORMAT *,UCHAR *,long);
static char *percent(char *, FORMAT *, va_list *);



/***********************************************************************/
/* read_number: Read a number from the input                           */
/*                                                                     */
/*     RETURNS: number                                                 */
/*                                                                     */
/***********************************************************************/

ULONG get_number(int hex, int digit)
{
 U8 buf[16]; 			// Input buffer
 U8 pos;			// current position in buffer 	
 U32 number;			// result number 
 U8 ch;				// current character 

 pos = 0;
 while(1)
  {
// Get a character when it is typed in.  Translate linefeed to CR. /////////////
   ch = get_byte() & 0x7F;

   if(
      ch == KEY_NL || 
      ch == KEY_CR
    ) 
    {
// new line or carreage return => terminate input //////////////////////////////
     break;
    }
   else
   if(
      (ch == KEY_BSP) || 
      (ch == KEY_RUBOUT)
    )
    {
// Handle backspace ////////////////////////////////////////////////////////////
     if(pos > 0)
      {
       put_byte('\b');
       put_byte(' ');
       put_byte('\b');
       pos--;
      }
    }
   else 
   if(ch <= KEY_CTRLZ)
    {
// Skip control characters (other than backspace and CR) ///////////////////////
     continue;
    }
   else
   if(pos + 1 >= 16)
    {
// Don't allow the input buffer to overflow. ///////////////////////////////////
     pos = 16 - 1;
     put_byte(7);
     break;
    }
   else 
    {
// write character to buffer ///////////////////////////////////////////////////
     buf[pos++] = ch;			// write byte to buffer 			
     put_byte(ch);			// write byte to console 

     if(digit != 0 && pos >= digit)
      {
// out of digit count ////////////////////////////////////////////////////////// 
       break;
      }
    }
  }

/* terminating char */
 buf[pos] = 0;				
 put_byte('\n');

// 
 pos = 0;

/* Skip leading spaces */
 while(buf[pos] == ' ')
  {
   ++pos;
  }


 if(hex == 0)
  {
   if(buf[pos] == '$')
    {
// process leading $ ///////////////////////////////////////////////////////////
     hex = 16;				// hexadecimal digit 
     pos += 1; 				// next position in buffer 
    }
   else
   if(buf[pos] == 'x')
    {
// x ///////////////////////////////////////////////////////////////////////////
     hex = 16;
     pos += 1;
    }
   else
   if(
      buf[pos]   == '0' && 
      (
       buf[pos+1] == 'x' || 
       buf[pos+1] == 'X'
      )
     )
    {
// 0x or 0X ////////////////////////////////////////////////////////////////////
     hex = 16;
     pos += 2;
    }    
   else
    {
     hex = 10;				// decimal number  
    } 
  }

/* Run through the string, adding the value represented by each        */
/* character to the total.                                             */

 number = 0;

// 
 while(buf[pos])
  {
   ch = buf[pos++];
   if ((ch >= '0') && (ch <= '9'))
    {
     ch -= '0';
    }
   else 
   if(hex == 16 && (ch >= 'A') && (ch <= 'F'))
    { 
     ch = ch - 'A' + 10;
    }
   else 
   if (hex == 16 && (ch >= 'a') && (ch <= 'f'))
    {
     ch = ch - 'a' + 10;
    }
   else
    {
     Print("Invalid number [%d] [%c]\n", ch, ch);
     return(0);
    }
 
   if(hex == 16)
    {
     number = (number << 4) + ch;
    }
   else
    {
     number = number * 10 + ch;
    }
  }

 return(number);
}


/***********************************************************************/
/*                                                                     */
/*   This subset of printf() supports the following conversion types:  */
/*                                                                     */
/*      %d, %i - output number as a signed decimal (this implementa-   */
/*               tion only uses the low 16 bits of the argument)       */
/*      %o - output number as an unsigned octal number                 */
/*      %x, %X - output number as unsigned hexadecimal                 */
/*      %c - output a single character                                 */
/*      %s - ouput a string                                            */
/*                                                                     */
/*   In addition, the normal width and precision specifiers may be     */
/*   used.                                                             */
/*                                                                     */
/***********************************************************************/

/***********************************************************************/
/*      strout: Output a formatted string, adding fill characters and  */
/*              field justification.                                   */
/*                                                                     */
/*      INPUTS: f_flags = ptr to format data structure                 */
/*              str = ptr to string                                    */
/*              len = length of string                                 */
/*                                                                     */
/***********************************************************************/

static void strout(FORMAT *f_flags, UCHAR *str, long len)
{
 long size;
 ULONG count = 0;
 UCHAR pad = ' ';
 int i;

#ifdef CHECK_POLLIO
 if(f_flags == 0)
  {
   return;
  } 
#endif

 if (len == 0 || str == 0)
  {
/* Check for a NULL string. */
   str = (UCHAR *)"(null)";
  }

/* Calculate the field width. */
 f_flags->count += len;
 size = (f_flags->width < len ) ? len : f_flags->width;
    
/* If the right justified field is set, fill the leading characters    */
/* with blanks. */

 if(f_flags->right)
  {
   for (i = size - len; i > 0; i--)
    {
     put_byte(pad);
     count++;
    }
   size = len;
  }

/* Output the string. */
 while (*str != '\0' && len > 0)
  {
   put_byte(*str);
   if(*str == '\n')
    put_byte('\r');

   str++;

   len--;
   size--;
 }

/* If left justified, it may be necessary to pad end of the string. */
 for(i = size - len; i > 0; i--)
  {
   put_byte(pad);
   count++;
  }

//
 f_flags->count += count;
}


/***********************************************************************/
/*     charout: Output a character                                     */
/*                                                                     */
/*      INPUTS: f_flags = ptr to format data structure                 */
/*              arg_pt = ptr to next argument in list                  */
/*                                                                     */
/***********************************************************************/

static void charout(FORMAT *f_flags, va_list *arg_pt)
{
 UCHAR cbuf[2];
 cbuf[0] = (UCHAR) va_arg((*arg_pt), ULONG);
 strout(f_flags, cbuf, 1);
}

/***********************************************************************/
/*      strlen: returns length of a string                             */
/*                                                                     */
/*      INPUTS: str = ptr to string being measured                     */
/*                                                                     */
/***********************************************************************/
									
static ULONG strlen(UCHAR *str)
{
 long i;
 for(i = 0; *str++; i++);
 return i;
}

/***********************************************************************/
/*   stringout: Output a string                                        */
/*                                                                     */
/*      INPUTS: f_flags = ptr to format data structure                 */
/*              arg_pt = ptr to next argument in list                  */
/*                                                                     */
/*        NOTE: Strings longer than 100 bytes are truncated!           */
/*                                                                     */
/***********************************************************************/
static void stringout(FORMAT *f_flags, va_list *arg_pt)
{
 UCHAR * str;
 ULONG len;
    
 str = (UCHAR *)va_arg((*arg_pt), int);
 len = strlen(str);
 if(len > f_flags->precision && f_flags->precision != 0)
  {
   str = str + (len - f_flags->precision);
   len = f_flags->precision;
  }
    
 strout(f_flags, str, len);
}

/***********************************************************************/
/*      decout: Output a value as a decimal number                     */
/*                                                                     */
/*      INPUTS: f_flags = ptr to format data structure                 */
/*              arg_pt = ptr to next argument in list                  */
/*                                                                     */
/***********************************************************************/

static void decout(FORMAT *f_flags, va_list *arg_pt)
{
 UCHAR dec_buff[20];
 UCHAR * pt;
 UCHAR neg;				/* value is negative */
 long  val;				/* abs value */	
 long  s_length;
 ULONG t_count;
 t_count = 0;

 /* Get the value to output from the argument list. */
 if(f_flags->mod == 'h')
  {
   val = (short) va_arg((*arg_pt), long);
  } 
 else
  {
   val = (long) va_arg((*arg_pt), long);
  }
    
    /*---------------------------------------------------------------------*/
    /* Null terminate the output string.  Not included in output count.    */
    /*---------------------------------------------------------------------*/

 pt = dec_buff + 19;			/* pointer last element in array */
 *(pt--) = '\0';			/* terminating char */

 if (val < 0)
  {
/* If val is negative, set the true flag and multiply by -1. */
   neg = LTRUE;
   val = -val;
  }
 else
  {
   neg = FALSE;
  }

/* While val is greater than zero, output characters to the string */
/* from right to left. */
    do
    {
     long i, tmp;

/* The next two statements cause problems on the 68000, since that */
/* processor does not have a 32-bit divide instruction.  They are  */
/* replaced by functionally equivalent routines. */

/*  *(pt--) = val % 10 + '0';  */
     tmp = val;
        
     while (tmp >= 10000) tmp -= 10000;
     while (tmp >= 1000) tmp -= 1000;
     while (tmp >= 100) tmp -= 100;
     while ((tmp -= 10) >= 10);
     if (tmp < 0) tmp += 10;

    *(pt--) = (UCHAR)(tmp + '0');
        
/*   val /= (short)10;        */
     if (val >= 10)
      {
       for (i = 0; val >= 100000; i += 10000) val -= 100000;
       for (     ; val >=  10000; i +=  1000) val -= 10000;
       for (     ; val >=   1000; i +=   100) val -= 1000;
       for (     ; val >=    100; i +=    10) val -= 100;
       for (     ; val >=     10; i++) val -= 10;
       val = i;
      }
     else
       val = 0;

     t_count++;
  } while (val != 0);

 while (t_count < f_flags->precision)
  {
   *(pt--) = '0';
    t_count++;
  }

    /*---------------------------------------------------------------------*/
    /* Next check to see if the 0 flag was used in the format and if the   */
    /* current count is less than the specified field width.  If TRUE then */
    /* fill in the leading zeros.  Temporarily leave room in the field for */
    /* an optional plus or minus sign.                                     */
    /*---------------------------------------------------------------------*/

 pt++;
 if (f_flags->zero)
  {
   s_length = strlen(pt);
   while (f_flags->width > s_length + 1)
    {
      *(--pt) = '0';
       s_length += 1;
    }
  }

    /*---------------------------------------------------------------------*/
    /* Add leading plus or minus sign, and/or leading zero if needed.      */
    /*---------------------------------------------------------------------*/
  
  if (neg)
       *(--pt) = '-';
  else if (f_flags->plus)
       *(--pt) = '+';
  else if (f_flags->space)
        *(--pt) = ' ';
  else if ((f_flags->zero) && (f_flags->width > s_length))
       *(--pt) = '0';
  
 f_flags->precision = DEFAULT;
 strout(f_flags, pt, strlen(pt));
}

/***********************************************************************/
/*      hexout: Output a value as a hexadecimal number                 */
/*                                                                     */
/*      INPUTS: f_flags = ptr to format data structure                 */
/*              arg_pt = ptr to next argument in list                  */
/*                                                                     */
/***********************************************************************/

static void hexout(FORMAT *f_flags, va_list *arg_pt)
{
    UCHAR c, *pt, d[16], hexbuff[16];
    ULONG val, t_count;
    long s_length;
    USHORT i;

    t_count = 0;
    
/* Set up the array of hex digits used in conversion. */
 for (i = 0; i <= 9; i++)
   d[i] = '0' + i;
 c = (f_flags->type == 'X') ? 'A' : 'a';
 for (i = 10; i <= 15; i++)
   d[i] = c++;
    
/* Obtain the value to be output. */
 if (f_flags->mod == 'h')
  {
   val = (USHORT) va_arg((*arg_pt), ULONG);
  }
 else
  {
   val = (ULONG) va_arg((*arg_pt), ULONG);
  }  

/* Null terminate string then process the value.*/

 pt = & hexbuff[15];
 *(pt--) = '\0';

/* Process the value */
 do
  {
   *(pt--) = d[val & 0xf];
    t_count++;
  } 
 while (val >>= 4);

 while (t_count < f_flags->precision)
  {
   *(pt--) = '0';
    t_count++;
  }

    /*---------------------------------------------------------------------*/
    /* Next check to see if the 0 flag was used in the format and if the   */
    /* current count is less than the specified field width.  If TRUE then */
    /* fill in the leading zeros.  Temporarily leave room in the field for */
    /* an optional plus or minus sign.                                     */
    /*---------------------------------------------------------------------*/
 pt++;

 if (f_flags->zero)
  {
   s_length = strlen(pt);
   if (f_flags->alter)
     while (f_flags->width > s_length+2)
      {
       *(--pt) = '0';
        s_length += 1;
      }
     else
      while (f_flags->width > s_length)
       {
        *(--pt) = '0';
        s_length += 1;
       }
  }

    /*---------------------------------------------------------------------*/
    /* If the "alternate form" flag is set, add 0x or 0X to the front of   */
    /* the string.                                                         */
    /*---------------------------------------------------------------------*/
    if (f_flags->alter)
    {
        if (f_flags->type == 'x')
            *(--pt) = 'x';
        else
            *(--pt) = 'X';
        *(--pt) = '0';
    }

 f_flags->precision = DEFAULT;
 strout(f_flags, pt, strlen(pt));
}

/************************************************************************/
/*     percent: Interpret a conversion specification        		*/
/*                                                                      */
/*      INPUTS: format = ptr to current position in format string       */
/*              arg_pt = ptr to next argument                           */
/*                                                                      */
/*     RETURNS: Ptr to next char in format string past this specifier.  */
/*     OUTPUTS: *f_flags = filled in with format specification          */
/*                                                                      */
/************************************************************************/

static char *percent(char *format, FORMAT *f_flags, va_list *arg_pt)
{
 UCHAR p_flag = FALSE;

/*  Initialize the f_flag structure with the default values. */

 f_flags->width     = DEFAULT;
 f_flags->precision = DEFAULT_P;
 f_flags->right     = LTRUE;		// '-' to set left justified
 f_flags->plus      = FALSE;		// '+'    result will always have sign (+ or -)
 f_flags->alter     = FALSE;		// '#'    for alternate form
 f_flags->zero      = FALSE;		// '0'    use leading zeros instead of spaces.
 f_flags->space     = FALSE;		// space  for leading space
 f_flags->type      = '\0';
 f_flags->mod       = '\0';		// l,L,h

 if (*format == '-')
  {
   f_flags->right = FALSE;
   format++;
  }

 if(*format == '+')
  {
   f_flags->plus = LTRUE;
   format++;
  }

 if(*format == ' ')
  {
   if (f_flags->plus == FALSE)
    f_flags->space = LTRUE;
   format++;
  }

 if(*format == '#')
  {
   f_flags->alter = LTRUE;
   format++;
  }

 if(*format == '0')
  {
   if (f_flags->right)
    f_flags->zero = LTRUE;
   format++;
  }

/************************************************************************/
/* Process the width specification of the format.  The width can be     */
/* specified by the next value in the arg_pt list.  If the next         */
/* character is '*' then get the width from arg_pt with a call to the   */
/* va_arg() macro.                                                      */
/*                                                                      */
/* The width can also be specified as a value in the format.   In this  */
/* case the width is specified by a decimal number.                    	*/
/************************************************************************/

 if (*format == '*')
  {
   f_flags->width = va_arg((*arg_pt), int);
   if (f_flags->width < 0)
    {
     f_flags->right = FALSE;
     f_flags->width = -f_flags->width;
    }
   format++;
  }
 else
  {
   if(*format >= '0' && *format <= '9')
    {
     f_flags->width = 0; 
     while(*format >= '0' && *format <= '9')
      {
       f_flags->width *= 10;
       f_flags->width += (*format++ - '0');
      }
    } 
  }

/************************************************************************/
/* Process the precision field.  The same options that are allowed for  */
/* the width field are permitted here.                              	*/
/************************************************************************/

if( *format == '.' )
 {
  p_flag = LTRUE;
  format++;
  if( *format == '*' )
   {
    f_flags->precision = va_arg((*arg_pt), int);
    format++;
   }
  else
   {
    f_flags->precision = 0;
    while(*format >= '0' && *format <= '9')
     {
      f_flags->precision *= 10;
      f_flags->precision += *format++ - '0';
     }
   }
 }

/************************************************************************/
/*  Check for process modifier characters. {h, l, L}                 	*/
/************************************************************************/

if((*format == 'h') || (*format == 'l') || (*format == 'L'))
  f_flags->mod = *format++;

/************************************************************************/
/* Based on the type of conversion it may be necessary to readjust the  */
/* precision flag to a default value of 0.                              */
/************************************************************************/
 if ((p_flag == FALSE) && (*format != 'c') && (*format != '%'))
  f_flags->precision = 0;

/************************************************************************/
/* The final step is to read in the conversion type.                    */
/************************************************************************/
 f_flags->type = *format++;
 return format;
}


/***********************************************************************/
/*      Print: Format output and send it to the console                */
/*                                                                     */
/*      INPUTS: format = ptr to format string                          */
/*              additional inputs as specified by *format.             */
/*                                                                     */
/*     RETURNS: # of characters output                                 */
/*        NOTE: Format strings are explained in the header at the      */
/*              top of this module.                                    */
/*                                                                     */
/***********************************************************************/
ULONG Print(char *format, ...)
{
 ULONG ip_addr;
 FORMAT f_flags;
 va_list arg_pt;

 va_start(arg_pt, format);

 f_flags.count = 0;

 if (format == NULL)
  format = (char *)"(null)";

    /*---------------------------------------------------------------------*/
    /* Process characters in 'format ' string until null terminator is     */
    /* reached.  If the character is not a '%' then simply print it.       */
    /* Otherwise it will require further processing.                       */
    /*---------------------------------------------------------------------*/

 while (*format != '\0')
  {
   if (*format != '%')
    {
     put_byte(*format);
     if (*format == '\n')
      put_byte('\r');
     format++;

     f_flags.count++;
     continue;
    }

/* Set up the fields in the format structure.*/
   format++;
   format = percent(format, &f_flags, &arg_pt);

/*-------------------------------------------------------------*/
/* Continue to process based on the format type:               */
/*  d,i   -   signed decimal                                   */
/*  x,X   -   unsigned hexadecimal                             */
/*  c     -   print least significant character of int         */
/*  s     -   argument taken to be (char *) pointer to string  */
/*  %     -   A % is printed. No argument is converted.        */
/*  I     -   IP address (dd.dd.dd.dd)                         */
/*-------------------------------------------------------------*/
   if(f_flags.type == 'd' || f_flags.type == 'i')
    {
     decout(&f_flags, &arg_pt);
    }
   else
   if(f_flags.type == 'x' || f_flags.type == 'X')
    {
     hexout(&f_flags, &arg_pt);
    }
   else
   if(f_flags.type == 'c')    
    {
     charout(&f_flags, &arg_pt);
    }
   else
   if(f_flags.type == 's') 
    {
     stringout(&f_flags, &arg_pt);
    }
   else
   if(f_flags.type == '%')
    {
     put_byte('%');
     f_flags.count++;
    }
   else
   if(f_flags.type == 'I')
    {
     ip_addr = va_arg((arg_pt), long);
      Print("%d.%d.%d.%d", (ip_addr >> 24) & 0xFF,
                         (ip_addr >> 16) & 0xFF,
                         (ip_addr >> 8)  & 0xFF,
                     ip_addr & 0xFF);
    }
  }

 return f_flags.count;
}

